"use strict";

global.nv_version = "5.0.0";

// 记录开始启动的时间
global.nv_start_timestamp = new Date();

import * as fs from 'fs'
import {fork} from 'child_process'
import define_global_methods from "./nv-modules/methods.js"
import add_init_hooks from "./nv-modules/init.js";

//读取命令行参数
import minimist from "minimist";
const args = minimist(process.argv)

// 判断 nv-content 是否被挂载
global._is_nv_content_folder_mounted = !fs.existsSync('./nv-content/__NOT_MOUNTED__');
global._is_nv_plugins_folder_mounted = !fs.existsSync('./nv-plugins/__NOT_MOUNTED__');
global._is_nv_themes_folder_mounted = !fs.existsSync('./nv-themes/__NOT_MOUNTED__');

if ( args.run == 'express' ) {
	// 子进程启动
	process.on('uncaughtException', function (err) {
		console.log(args.docker ? '[ 错误 ]' : `\x1B[31m\x1B[1m[ 错误 ]：\x1B[0m`, err)
	});

	define_global_methods();

	//内置的init钩子
	add_init_hooks();

	do_async_action('async:_nv_init');
} else {
	// 主进程启动
	args.noclear !== 'true' && console.clear()
	console.log(args.docker ? `nvPress ${global.nv_version}\n主进程已启动\n` : `\x1B[36mnvPress ${global.nv_version}\x1B[0m\n\x1B[2m主进程已启动\n\x1B[0m`)
	//带参数启动子进程、监听子进程、重启子进程
	var handle_server = ()=>{
		var express = fork( import.meta.filename ,['--run=express',`--port=${args.port || ''}`, `--docker=${args.docker || ''}`])
		
		express.on('message', (args)=>{
			if (args == 'reboot_nvPress') {
				express.kill();
				console.log('主进程重启Express');
				handle_server();
			}
			else if (args == 'turnoff_nvPress') {
				express.kill();
				console.log('网络服务已关闭');
				process.exit();
			}
		});

		// 在Docker中运行时，总是主进程收到 SIGTERM 信号。阻止关闭延迟9.5秒！
		// 告诉子进程，让子进程来处理关闭过程
		if (args.docker == 'true') {
			process.on('SIGTERM',()=>{
				console.log('Docker发起关闭命令')
				express.send('Docker_Close_Container');
				setTimeout(()=>{},9500)
			})
		}
	}
	handle_server();
}

